# Mesures de réseaux d'interaction

Nous allons analyser un réseau de collaboration scientifique en informatique. Le réseau est extrait de DBLP et disponible sur [SNAP](https://snap.stanford.edu/data/com-DBLP.html).

GraphStream permet de mesurer de nombreuses caractéristiques d'un réseau. La plupart de ces mesures sont implantées comme des méthodes statiques dans la classe [`Toolkit`](https://data.graphstream-project.org/api/gs-algo/current/org/graphstream/algorithm/Toolkit.html). Elles vous seront très utiles par la suite.

1. Commencez par télécharger les données et les lire avec GraphStream. GraphStream sait lire ce format. Voir [`FileSourceEdge`](https://data.graphstream-project.org/api/gs-core/current/org/graphstream/stream/file/FileSourceEdge.html) et ce [tutoriel](http://graphstream-project.org/doc/Tutorials/Reading-files-using-FileSource/). Vous pouvez essayer de visualiser le graphe mais pour cette taille ça sera très lent et très peu parlant.

2. Prenez quelques mesures de base: nombre de nœuds et de liens, degré moyen, coefficient de clustering. Quel sera le coefficient de clustering pour un réseau aléatoire de la même taille et du même degré moyen ?

    Quelques mestures faites sur le graphe :


    | Mesures             |     Résultats     |  Functions |
    | :------------------ | :---------------: | -------------: |
    | $`N`$                 | 317080            | [`graph.getNodeCount()`](https://data.graphstream-project.org/api/gs-core/current/org/graphstream/graph/Structure.html#getNodeCount()) |
    | $`L`$                 | 1049866           | [`graph.getEdgeCount()`](https://data.graphstream-project.org/api/gs-core/current/org/graphstream/graph/Structure.html#getEdgeCount()) |
    | $`\langle k \rangle`$ | 6,6220889        | [`Toolkit.averageDegree(graph)`](https://data.graphstream-project.org/api/gs-algo/current/org/graphstream/algorithm/Toolkit.html#averageDegree(org.graphstream.graph.Graph)) |
    | $`\langle C \rangle`$ | 0.6324308 | [`Toolkit.averageClusteringCoefficient(graph)`](https://data.graphstream-project.org/api/gs-algo/current/org/graphstream/algorithm/Toolkit.html#averageClusteringCoefficient(org.graphstream.graph.Graph)) |

    Le coefficient de clustering $`Ci`$ mesure la probabilité $`p`$ qu'un noeud $`i`$ soit relié à un autre noeud du réseau.  
    Dans le cas d'un réseau aléatoire $`G(N,p)`$, la probabilité est la même pour tous les noeuds. Ce qui veut dire que $`C_i=p`$.
    
    Avec $`p=\frac{\langle k \rangle}{N}`$, le coefficient de clustering moyen d'un graphe aléatoire est égal à $`\frac{6,622088}{317080}=2.088 \times 10^{-5}`$

3. Le réseau est-il connexe ? Un réseau aléatoire de la même taille et degré moyen sera-t-il connexe ? À partir de quel degré moyen un réseau aléatoire avec cette taille devient connexe ?

    GraphStream permet de vérifier si un graphe est connexe avec [`Toolkit.isConnected()`](https://data.graphstream-project.org/api/gs-algo/current/org/graphstream/algorithm/Toolkit.html#isConnected(org.graphstream.graph.Graph)). Dans notre cas le réseau est connexe car l'algorithme parcours bien chaque noeud une seule fois en passant par leurs voisins.

    Sans parcourir un réseau aléatoire, on peut déduire qu'il est connexe à partir du moment où $`1 \gt \frac{\langle k \rangle}{\log{N}}`$.

    Dans notre cas cela donne $`\frac{6,622088}{\log 317080}`$

4. Calculez la distribution des degrés et tracez-la avec `gnuplot` (ou avec votre outil préféré) d'abord en échelle linéaire, ensuite en échelle log-log. Est-ce qu'on observe une ligne droite en log-log ? Que cela nous indique ? Tracez la distribution de Poisson avec la même moyenne pour comparaison. Utilisez la commande `fit` de `gnuplot` pour trouver les coefficients de la loi de puissance et tracez-la.

    La distribution de degrés $`p_k = \frac{N_k}{N}`$ est la probabilité qu'un nœud choisi au hasard ait degré $`k`$. On peut utiliser [`Toolkit.degreeDistribution()`](https://data.graphstream-project.org/api/gs-algo/current/org/graphstream/algorithm/Toolkit.html#degreeDistribution(org.graphstream.graph.Graph)) pour obtenir $`N_k`$ et normaliser par la suite :

    ```java
    int[] dd = Toolkit.degreeDistribution(graph);
    for (int k = 0; k < dd.length; k++) {
      if (dd[k] != 0) {
        System.out.printf(Locale.US, "%6d%20.8f%n", k, (double)dd[k] / graph.getNodeCount());
      }
    }
    ```

    En traçant la distribution de degrés en échelle log-log on observe une ligne droite pendant plusieurs ordres de grandeur. Cela nous indique une loi de puissance :

    
    $`p_k = C k^{-\gamma}`$
    

    On utilise ce [script](/script/gen_degree.gnu) pour tracer la distribution et estimer l'exposant de la loi de puissance.

    ![distribution des degrés](/image/ddegree_dblp.png)

    On a $`\gamma = 2.7 \pm 0.04`$

5. Maintenant on va calculer la distance moyenne dans le réseau. Le calcul des plus courts chemins entre toutes les paires de nœuds prendra plusieurs heures pour cette taille de réseau. C'est pourquoi on va estimer la distance moyenne par échantillonnage en faisant un parcours en largeur à partir de 1000 sommets choisis au hasard. L'hypothèse des six degrés de séparation se confirme-t-elle ? Est-ce qu'il s'agit d'un réseau petit monde ? Quelle sera la distance moyenne dans un réseau aléatoire avec les mêmes caractéristiques ? Tracez également la *distribution* des distances. Formulez une hypothèse sur la loi de cette distribution.

    **TODO**
    On utilise ce [script](/script/gen_distance.gnu) pour tracer la distribution.

    ![distribution des distances](/image/ddistance_dblp.png)

6. Utilisez les générateurs de GraphStream pour générer un réseau aléatoire et un réseau avec la méthode d'attachement préférentiel (Barabasi-Albert) qui ont la même taille et le même degré moyen. Refaites les mesures des questions précédentes pour ces deux réseaux. Les résultats expérimentaux correspondent-ils aux prédictions théoriques ? Comparez avec le réseau de collaboration. Que peut-on conclure ?

    **TODO**
    ![distribution des distances](/image/ddistance_random.png)
    ![distribution des distances](/image/ddistance_preferential.png)
    ![distribution des degrés](/image/ddegree_random.png)
    ![distribution des degrés](/image/ddegree_preferential.png)